@@ -6,9 +6,8 @@ use attributes::Attribute;
 use coord_units::CoordUnits;
 use drawing_ctx::DrawingCtx;
 use error::RenderingError;
-use handle::RsvgHandle;
 use node::{NodeResult, NodeTrait, RsvgNode};
-use parsers::parse;
+use parsers::ParseValue;
 use property_bag::PropertyBag;
 
 coord_units!(ClipPathUnits, CoordUnits::UserSpaceOnUse);
@@ -79,11 +78,10 @@ impl NodeClipPath {
 }
 
 impl NodeTrait for NodeClipPath {
-    fn set_atts(&self, _: &RsvgNode, _: *const RsvgHandle, pbag: &PropertyBag<'_>) -> NodeResult {
+    fn set_atts(&self, _: &RsvgNode, pbag: &PropertyBag<'_>) -> NodeResult {
         for (attr, value) in pbag.iter() {
             match attr {
-                Attribute::ClipPathUnits => self.units.set(parse("clipPathUnits", value, ())?),
-
+                Attribute::ClipPathUnits => self.units.set(attr.parse(value, ())?),
                 _ => (),
             }
         }
