@@ -1,9 +1,11 @@
 use std::cell::Cell;
 
-extern "C" {
-    fn rsvg_get_default_dpi_y() -> f64;
-    fn rsvg_get_default_dpi_x() -> f64;
-}
+// This is configurable at runtime
+const DEFAULT_DPI_X: f64 = 90.0;
+const DEFAULT_DPI_Y: f64 = 90.0;
+
+static mut DPI_X: f64 = DEFAULT_DPI_X;
+static mut DPI_Y: f64 = DEFAULT_DPI_Y;
 
 #[derive(Debug, Clone, Default)]
 pub struct Dpi {
@@ -14,7 +16,7 @@ pub struct Dpi {
 impl Dpi {
     pub fn x(&self) -> f64 {
         if self.x.get() <= 0.0 {
-            unsafe { rsvg_get_default_dpi_x() }
+            unsafe { DPI_X }
         } else {
             self.x.get()
         }
@@ -26,7 +28,7 @@ impl Dpi {
 
     pub fn y(&self) -> f64 {
         if self.y.get() <= 0.0 {
-            unsafe { rsvg_get_default_dpi_y() }
+            unsafe { DPI_Y }
         } else {
             self.y.get()
         }
@@ -36,3 +38,18 @@ impl Dpi {
         self.y.set(dpi_y)
     }
 }
+
+#[no_mangle]
+pub unsafe extern "C" fn rsvg_rust_set_default_dpi_x_y(dpi_x: f64, dpi_y: f64) {
+    if dpi_x <= 0.0 {
+        DPI_X = DEFAULT_DPI_X;
+    } else {
+        DPI_X = dpi_x;
+    }
+
+    if dpi_y <= 0.0 {
+        DPI_Y = DEFAULT_DPI_Y;
+    } else {
+        DPI_Y = dpi_y;
+    }
+}
