@@ -2,9 +2,6 @@ use cairo;
 use cairo::MatrixTrait;
 use cairo_sys;
 use glib::translate::*;
-use pango::{self, ContextExt, FontMapExt, LayoutExt};
-use pango_sys;
-use pangocairo;
 use std::cell::RefCell;
 use std::rc::{Rc, Weak};
 
@@ -15,12 +12,12 @@ use defs::Fragment;
 use dpi::Dpi;
 use error::RenderingError;
 use filters;
-use float_eq_cairo::ApproxEqCairo;
-use handle::{self, RsvgHandle};
+use gradient::NodeGradient;
 use length::Dasharray;
 use mask::NodeMask;
 use node::{CascadedValues, NodeType, RsvgNode};
-use paint_server::{self, PaintServer};
+use paint_server::{PaintServer, PaintSource};
+use pattern::NodePattern;
 use rect::RectangleExt;
 use state::{
     ClipRule,
@@ -32,9 +29,9 @@ use state::{
     StrokeDasharray,
     StrokeLinecap,
     StrokeLinejoin,
-    TextRendering,
 };
-use unitinterval::UnitInterval;
+use svg::Svg;
+use unit_interval::UnitInterval;
 use viewbox::ViewBox;
 
 /// Holds values that are required to normalize `Length` values to a current viewport.
@@ -95,7 +92,7 @@ impl Drop for ViewParams {
 }
 
 pub struct DrawingCtx {
-    handle: *const RsvgHandle,
+    svg: Rc<Svg>,
 
     rect: cairo::Rectangle,
     dpi: Dpi,
@@ -127,19 +124,19 @@ pub struct DrawingCtx {
 
     acquired_nodes: Rc<RefCell<Vec<RsvgNode>>>,
 
-    is_testing: bool,
+    testing: bool,
 }
 
 impl DrawingCtx {
     pub fn new(
-        handle: *const RsvgHandle,
+        svg: Rc<Svg>,
         cr: &cairo::Context,
         width: f64,
         height: f64,
         vb_width: f64,
         vb_height: f64,
         dpi: Dpi,
-        is_testing: bool,
+        testing: bool,
     ) -> DrawingCtx {
         let mut affine = cr.get_matrix();
         let rect = cairo::Rectangle {
@@ -166,7 +163,7 @@ impl DrawingCtx {
         view_box_stack.push(ViewBox::new(0.0, 0.0, vb_width, vb_height));
 
         DrawingCtx {
-            handle,
+            svg: svg.clone(),
             rect,
             dpi,
             num_elements_rendered_through_use: 0,
@@ -179,10 +176,14 @@ impl DrawingCtx {
             bbox_stack: Vec::new(),
             drawsub_stack: Vec::new(),
             acquired_nodes: Rc::new(RefCell::new(Vec::new())),
-            is_testing,
+            testing,
         }
     }
 
+    pub fn is_testing(&self) -> bool {
+        self.testing
+    }
+
     pub fn get_cairo_context(&self) -> cairo::Context {
         self.cr.clone()
     }
@@ -196,14 +197,6 @@ impl DrawingCtx {
         self.cr = cr.clone();
     }
 
-    pub fn is_cairo_context_nested(&self, cr: &cairo::Context) -> bool {
-        cr.to_raw_none() != self.initial_cr.to_raw_none()
-    }
-
-    pub fn get_cr_stack(&self) -> &Vec<cairo::Context> {
-        &self.cr_stack
-    }
-
     pub fn get_width(&self) -> f64 {
         self.rect.width
     }
@@ -212,18 +205,6 @@ impl DrawingCtx {
         self.rect.height
     }
 
-    pub fn get_raw_offset(&self) -> (f64, f64) {
-        (self.rect.x, self.rect.y)
-    }
-
-    pub fn get_offset(&self) -> (f64, f64) {
-        if self.is_cairo_context_nested(&self.get_cairo_context()) {
-            (0.0, 0.0)
-        } else {
-            (self.rect.x, self.rect.y)
-        }
-    }
-
     /// Gets the viewport that was last pushed with `push_view_box()`.
     pub fn get_view_params(&self) -> ViewParams {
         let view_box_stack = self.view_box_stack.borrow();
@@ -282,7 +263,7 @@ impl DrawingCtx {
     // acquire it again.  If you acquire a node "#foo" and don't release it before
     // trying to acquire "foo" again, you will obtain a %NULL the second time.
     pub fn get_acquired_node(&mut self, fragment: &Fragment) -> Option<AcquiredNode> {
-        if let Some(node) = handle::lookup_node(self.handle, fragment) {
+        if let Some(node) = self.svg.lookup(fragment) {
             if !self.acquired_nodes_contains(&node) {
                 self.acquired_nodes.borrow_mut().push(node.clone());
                 let acq = AcquiredNode(self.acquired_nodes.clone(), node.clone());
@@ -331,6 +312,18 @@ impl DrawingCtx {
             })
     }
 
+    fn is_cairo_context_nested(&self, cr: &cairo::Context) -> bool {
+        cr.to_raw_none() != self.initial_cr.to_raw_none()
+    }
+
+    fn get_offset(&self) -> (f64, f64) {
+        if self.is_cairo_context_nested(&self.get_cairo_context()) {
+            (0.0, 0.0)
+        } else {
+            (self.rect.x, self.rect.y)
+        }
+    }
+
     pub fn with_discrete_layer(
         &mut self,
         node: &RsvgNode,
@@ -519,148 +512,83 @@ impl DrawingCtx {
         }
     }
 
-    pub fn get_pango_context(&self) -> pango::Context {
-        let font_map = pangocairo::FontMap::get_default().unwrap();
-        let context = font_map.create_context().unwrap();
-        let cr = self.get_cairo_context();
-        pangocairo::functions::update_context(&cr, &context);
-
-        // Pango says this about pango_cairo_context_set_resolution():
-        //
-        //     Sets the resolution for the context. This is a scale factor between
-        //     points specified in a #PangoFontDescription and Cairo units. The
-        //     default value is 96, meaning that a 10 point font will be 13
-        //     units high. (10 * 96. / 72. = 13.3).
-        //
-        // I.e. Pango font sizes in a PangoFontDescription are in *points*, not pixels.
-        // However, we are normalizing everything to userspace units, which amount to
-        // pixels.  So, we will use 72.0 here to make Pango not apply any further scaling
-        // to the size values we give it.
-        //
-        // An alternative would be to divide our font sizes by (dpi_y / 72) to effectively
-        // cancel out Pango's scaling, but it's probably better to deal with Pango-isms
-        // right here, instead of spreading them out through our Length normalization
-        // code.
-        pangocairo::functions::context_set_resolution(&context, 72.0);
-
-        if self.is_testing {
-            let mut options = cairo::FontOptions::new();
-
-            options.set_antialias(cairo::Antialias::Gray);
-            options.set_hint_style(cairo::enums::HintStyle::Full);
-            options.set_hint_metrics(cairo::enums::HintMetrics::On);
-
-            pangocairo::functions::context_set_font_options(&context, &options);
-        }
-
-        context
-    }
-
-    pub fn draw_pango_layout(
-        &mut self,
-        layout: &pango::Layout,
-        values: &ComputedValues,
-        x: f64,
-        y: f64,
-        clipping: bool,
-    ) -> Result<(), RenderingError> {
-        let (ink, _) = layout.get_extents();
-
-        if ink.width == 0 || ink.height == 0 {
-            return Ok(());
-        }
-
-        let cr = self.get_cairo_context();
-        cr.save();
-
-        self.set_affine_on_cr(&cr);
-
-        let affine = cr.get_matrix();
-
-        let gravity = layout.get_context().unwrap().get_gravity();
-        let bbox = compute_text_bbox(&ink, x, y, &affine, gravity);
-
-        if !clipping {
-            self.insert_bbox(&bbox);
-        }
-
-        cr.set_antialias(cairo::Antialias::from(values.text_rendering));
-
-        self.setup_cr_for_stroke(&cr, values);
-
-        let rotation = unsafe { pango_sys::pango_gravity_to_rotation(gravity.to_glib()) };
-
-        cr.move_to(x, y);
-        if !rotation.approx_eq_cairo(&0.0) {
-            cr.rotate(-rotation);
-        }
-
-        let current_color = &values.color.0;
-
-        let fill_opacity = &values.fill_opacity.0;
-
-        let res = if !clipping {
-            paint_server::set_source_paint_server(
-                self,
-                &values.fill.0,
-                fill_opacity,
-                &bbox,
-                current_color,
-            )
-            .and_then(|had_paint_server| {
-                if had_paint_server {
-                    pangocairo::functions::update_layout(&cr, layout);
-                    pangocairo::functions::show_layout(&cr, layout);
-                };
-                Ok(())
-            })
-        } else {
-            Ok(())
+    fn set_color(
+        &self,
+        color: &cssparser::Color,
+        opacity: &UnitInterval,
+        current_color: &cssparser::RGBA,
+    ) {
+        let rgba = match *color {
+            cssparser::Color::RGBA(ref rgba) => rgba,
+            cssparser::Color::CurrentColor => current_color,
         };
 
-        if res.is_ok() {
-            let stroke_opacity = &values.stroke_opacity.0;
-
-            let mut need_layout_path = clipping;
-
-            let res = if !clipping {
-                paint_server::set_source_paint_server(
-                    self,
-                    &values.stroke.0,
-                    stroke_opacity,
-                    &bbox,
-                    &current_color,
-                )
-                .and_then(|had_paint_server| {
-                    if had_paint_server {
-                        need_layout_path = true;
-                    }
-                    Ok(())
-                })
-            } else {
-                Ok(())
-            };
-
-            if res.is_ok() {
-                if need_layout_path {
-                    pangocairo::functions::update_layout(&cr, layout);
-                    pangocairo::functions::layout_path(&cr, layout);
-
-                    if !clipping {
-                        let ib = BoundingBox::new(&affine).with_ink_extents(cr.stroke_extents());
-                        cr.stroke();
-                        self.insert_bbox(&ib);
-                    }
-                }
-            }
-        }
-
-        cr.restore();
-
-        res
+        let &UnitInterval(o) = opacity;
+        self.get_cairo_context().set_source_rgba(
+            f64::from(rgba.red_f32()),
+            f64::from(rgba.green_f32()),
+            f64::from(rgba.blue_f32()),
+            f64::from(rgba.alpha_f32()) * o,
+        );
     }
 
-    fn setup_cr_for_stroke(&self, cr: &cairo::Context, values: &ComputedValues) {
+    pub fn set_source_paint_server(
+        &mut self,
+        ps: &PaintServer,
+        opacity: &UnitInterval,
+        bbox: &BoundingBox,
+        current_color: &cssparser::RGBA,
+    ) -> Result<bool, RenderingError> {
+        let mut had_paint_server;
+
+        match *ps {
+            PaintServer::Iri {
+                ref iri,
+                ref alternate,
+            } => {
+                had_paint_server = false;
+
+                if let Some(acquired) = self.get_acquired_node(iri) {
+                    let node = acquired.get();
+
+                    if node.get_type() == NodeType::LinearGradient
+                        || node.get_type() == NodeType::RadialGradient
+                    {
+                        had_paint_server = node.with_impl(|n: &NodeGradient| {
+                            n.resolve_fallbacks_and_set_pattern(&node, self, opacity, bbox)
+                        })?;
+                    } else if node.get_type() == NodeType::Pattern {
+                        had_paint_server = node.with_impl(|n: &NodePattern| {
+                            n.resolve_fallbacks_and_set_pattern(&node, self, opacity, bbox)
+                        })?;
+                    }
+                }
+
+                if !had_paint_server && alternate.is_some() {
+                    self.set_color(alternate.as_ref().unwrap(), opacity, current_color);
+                    had_paint_server = true;
+                } else {
+                    rsvg_log!(
+                        "pattern \"{}\" was not found and there was no fallback alternate",
+                        iri
+                    );
+                }
+            }
+
+            PaintServer::SolidColor(color) => {
+                self.set_color(&color, opacity, current_color);
+                had_paint_server = true;
+            }
+
+            PaintServer::None => {
+                had_paint_server = false;
+            }
+        };
+
+        Ok(had_paint_server)
+    }
+
+    pub fn setup_cr_for_stroke(&self, cr: &cairo::Context, values: &ComputedValues) {
         let params = self.get_view_params();
 
         cr.set_line_width(values.stroke_width.0.normalize(values, &params));
@@ -705,41 +633,35 @@ impl DrawingCtx {
 
         let fill_opacity = &values.fill_opacity.0;
 
-        let res = paint_server::set_source_paint_server(
-            self,
-            &values.fill.0,
-            fill_opacity,
-            &bbox,
-            current_color,
-        )
-        .and_then(|had_paint_server| {
-            if had_paint_server {
-                if values.stroke.0 == PaintServer::None {
-                    cr.fill();
-                } else {
-                    cr.fill_preserve();
-                }
-            }
-
-            Ok(())
-        })
-        .and_then(|_| {
-            let stroke_opacity = values.stroke_opacity.0;
-
-            paint_server::set_source_paint_server(
-                self,
-                &values.stroke.0,
-                &stroke_opacity,
-                &bbox,
-                &current_color,
-            )
+        let res = self
+            .set_source_paint_server(&values.fill.0, fill_opacity, &bbox, current_color)
             .and_then(|had_paint_server| {
                 if had_paint_server {
-                    cr.stroke();
+                    if values.stroke.0 == PaintServer::None {
+                        cr.fill();
+                    } else {
+                        cr.fill_preserve();
+                    }
                 }
+
                 Ok(())
             })
-        });
+            .and_then(|_| {
+                let stroke_opacity = values.stroke_opacity.0;
+
+                self.set_source_paint_server(
+                    &values.stroke.0,
+                    &stroke_opacity,
+                    &bbox,
+                    &current_color,
+                )
+                .and_then(|had_paint_server| {
+                    if had_paint_server {
+                        cr.stroke();
+                    }
+                    Ok(())
+                })
+            });
 
         // clear the path in case stroke == fill == None; otherwise
         // we leave it around from computing the bounding box
@@ -762,7 +684,7 @@ impl DrawingCtx {
         cr.set_matrix(matrix);
     }
 
-    pub fn clip(&mut self, x: f64, y: f64, w: f64, h: f64) {
+    pub fn clip(&self, x: f64, y: f64, w: f64, h: f64) {
         let cr = self.get_cairo_context();
         let save_affine = cr.get_matrix();
 
@@ -773,6 +695,24 @@ impl DrawingCtx {
         cr.set_matrix(save_affine);
     }
 
+    pub fn get_snapshot(&self, surface: &cairo::ImageSurface) {
+        let (x, y) = (self.rect.x, self.rect.y);
+
+        // TODO: as far as I can tell this should not render elements past the last (topmost) one
+        // with enable-background: new (because technically we shouldn't have been caching them).
+        // Right now there are no enable-background checks whatsoever.
+        let cr = cairo::Context::new(&surface);
+        for draw in self.cr_stack.iter() {
+            let nested = self.is_cairo_context_nested(&draw);
+            cr.set_source_surface(
+                &draw.get_target(),
+                if nested { 0f64 } else { -x },
+                if nested { 0f64 } else { -y },
+            );
+            cr.paint();
+        }
+    }
+
     pub fn draw_node_on_surface(
         &mut self,
         node: &RsvgNode,
@@ -840,6 +780,15 @@ impl DrawingCtx {
         res
     }
 
+    pub fn mask_surface(&mut self, mask: &cairo::ImageSurface) {
+        let cr = self.get_cairo_context();
+
+        cr.identity_matrix();
+
+        let (xofs, yofs) = self.get_offset();
+        cr.mask_surface(&mask, xofs, yofs);
+    }
+
     pub fn add_node_and_ancestors_to_stack(&mut self, node: &RsvgNode) {
         self.drawsub_stack.push(node.clone());
         if let Some(ref parent) = node.get_parent() {
@@ -860,49 +809,6 @@ impl DrawingCtx {
     }
 }
 
-// FIXME: should the pango crate provide this like PANGO_GRAVITY_IS_VERTICAL() ?
-fn gravity_is_vertical(gravity: pango::Gravity) -> bool {
-    match gravity {
-        pango::Gravity::East | pango::Gravity::West => true,
-        _ => false,
-    }
-}
-
-fn compute_text_bbox(
-    ink: &pango::Rectangle,
-    x: f64,
-    y: f64,
-    affine: &cairo::Matrix,
-    gravity: pango::Gravity,
-) -> BoundingBox {
-    let pango_scale = f64::from(pango::SCALE);
-
-    let ink_x = f64::from(ink.x);
-    let ink_y = f64::from(ink.y);
-    let ink_width = f64::from(ink.width);
-    let ink_height = f64::from(ink.height);
-
-    let rect = if gravity_is_vertical(gravity) {
-        cairo::Rectangle {
-            x: x + (ink_x - ink_height) / pango_scale,
-            y: y + ink_y / pango_scale,
-            width: ink_height / pango_scale,
-            height: ink_width / pango_scale,
-        }
-    } else {
-        cairo::Rectangle {
-            x: x + ink_x / pango_scale,
-            y: y + ink_y / pango_scale,
-            width: ink_width / pango_scale,
-            height: ink_height / pango_scale,
-        }
-    };
-
-    BoundingBox::new(affine)
-        .with_rect(Some(rect))
-        .with_ink_rect(Some(rect))
-}
-
 fn compute_stroke_and_fill_box(cr: &cairo::Context, values: &ComputedValues) -> BoundingBox {
     let affine = cr.get_matrix();
 
@@ -1035,17 +941,6 @@ impl From<ShapeRendering> for cairo::Antialias {
     }
 }
 
-impl From<TextRendering> for cairo::Antialias {
-    fn from(tr: TextRendering) -> cairo::Antialias {
-        match tr {
-            TextRendering::Auto
-            | TextRendering::OptimizeLegibility
-            | TextRendering::GeometricPrecision => cairo::Antialias::Default,
-            TextRendering::OptimizeSpeed => cairo::Antialias::None,
-        }
-    }
-}
-
 #[derive(Default, Clone, Copy, Debug, PartialEq)]
 #[repr(C)]
 pub struct RsvgRectangle {
