@@ -153,7 +153,9 @@ pub enum LoadingError {
     XmlParseError(String),
     // Could not parse data: URL
     CouldNotCreateXmlParser,
+    BadUrl,
     BadDataUrl,
+    BadCss,
     Cairo(cairo::Status),
     EmptyData,
     SvgHasNoElements,
@@ -168,7 +170,9 @@ impl error::Error for LoadingError {
             LoadingError::NoDataPassedToParser => "no data passed to parser",
             LoadingError::CouldNotCreateXmlParser => "could not create XML parser",
             LoadingError::XmlParseError(_) => "XML parse error",
+            LoadingError::BadUrl => "invalid URL",
             LoadingError::BadDataUrl => "invalid data: URL",
+            LoadingError::BadCss => "invalid CSS",
             LoadingError::Cairo(_) => "cairo error",
             LoadingError::EmptyData => "empty data",
             LoadingError::SvgHasNoElements => "SVG has no elements",
@@ -186,7 +190,9 @@ impl fmt::Display for LoadingError {
             LoadingError::XmlParseError(ref s) => write!(f, "XML parse error: {}", s),
             LoadingError::NoDataPassedToParser
             | LoadingError::CouldNotCreateXmlParser
+            | LoadingError::BadUrl
             | LoadingError::BadDataUrl
+            | LoadingError::BadCss
             | LoadingError::EmptyData
             | LoadingError::SvgHasNoElements
             | LoadingError::RootElementIsNotSvg
