@@ -1,17 +1,15 @@
 use std::cell::{Cell, RefCell};
-use std::ptr;
 
 use cairo::{self, ImageSurface, MatrixTrait, PatternTrait};
 
-use allowed_url::AllowedUrl;
 use aspect_ratio::AspectRatio;
 use attributes::Attribute;
 use defs::{Fragment, Href};
 use drawing_ctx::DrawingCtx;
 use error::{NodeError, RenderingError};
-use handle::{self, RsvgHandle};
+use handle::{self, LoadOptions};
 use node::{CascadedValues, NodeResult, NodeTrait, RsvgNode};
-use parsers::{parse, ParseError};
+use parsers::{ParseError, ParseValue};
 use property_bag::PropertyBag;
 use rect::IRect;
 use surface_utils::shared_surface::{SharedImageSurface, SurfaceType};
@@ -25,10 +23,7 @@ pub struct Image {
     base: Primitive,
     aspect: Cell<AspectRatio>,
     href: RefCell<Option<Href>>,
-
-    // Storing this here seems hack-ish... It's required by rsvg_cairo_surface_new_from_href(). The
-    // <image> element calls it in set_atts() but I don't think it belongs there.
-    handle: Cell<*const RsvgHandle>,
+    load_options: RefCell<Option<LoadOptions>>,
 }
 
 impl Image {
@@ -39,8 +34,7 @@ impl Image {
             base: Primitive::new::<Self>(),
             aspect: Cell::new(AspectRatio::default()),
             href: RefCell::new(None),
-
-            handle: Cell::new(ptr::null()),
+            load_options: RefCell::new(None),
         }
     }
 
@@ -131,11 +125,10 @@ impl Image {
             unreachable!();
         };
 
-        let aurl = AllowedUrl::from_href(url, handle::get_base_url(self.handle.get()).as_ref())
-            .map_err(|_| FilterError::InvalidInput)?;
+        let load_options_ref = self.load_options.borrow();
 
         // FIXME: translate the error better here
-        let surface = handle::load_image_to_surface(self.handle.get() as *mut _, &aurl)
+        let surface = handle::load_image_to_surface(load_options_ref.as_ref().unwrap(), &url)
             .map_err(|_| FilterError::InvalidInput)?;
 
         let output_surface = ImageSurface::create(
@@ -187,19 +180,12 @@ impl Image {
 }
 
 impl NodeTrait for Image {
-    fn set_atts(
-        &self,
-        node: &RsvgNode,
-        handle: *const RsvgHandle,
-        pbag: &PropertyBag<'_>,
-    ) -> NodeResult {
-        self.base.set_atts(node, handle, pbag)?;
+    fn set_atts(&self, node: &RsvgNode, pbag: &PropertyBag<'_>) -> NodeResult {
+        self.base.set_atts(node, pbag)?;
 
         for (attr, value) in pbag.iter() {
             match attr {
-                Attribute::PreserveAspectRatio => {
-                    self.aspect.set(parse("preserveAspectRatio", value, ())?)
-                }
+                Attribute::PreserveAspectRatio => self.aspect.set(attr.parse(value, ())?),
 
                 // "path" is used by some older Adobe Illustrator versions
                 Attribute::XlinkHref | Attribute::Path => self.set_href(attr, value)?,
@@ -208,7 +194,11 @@ impl NodeTrait for Image {
             }
         }
 
-        self.handle.set(handle);
+        Ok(())
+    }
+
+    fn resolve_resources(&self, load_options: &LoadOptions) -> NodeResult {
+        *self.load_options.borrow_mut() = Some(load_options.clone());
 
         Ok(())
     }
