@@ -7,22 +7,21 @@ use attributes::Attribute;
 use bbox::*;
 use coord_units::CoordUnits;
 use defs::Fragment;
-use drawing_ctx::{AcquiredNode, DrawingCtx};
+use drawing_ctx::{AcquiredNode, DrawingCtx, NodeStack};
 use error::*;
-use handle::RsvgHandle;
 use length::*;
 use node::*;
 use paint_server::PaintSource;
-use parsers::{parse, Parse, ParseError};
+use parsers::{Parse, ParseError, ParseValue};
 use property_bag::PropertyBag;
 use rect::RectangleExt;
 use state::{ComputedValues, StopColor};
 use stop::*;
-use unitinterval::UnitInterval;
+use unit_interval::UnitInterval;
 
 #[derive(Copy, Clone)]
 pub struct ColorStop {
-    pub offset: f64,
+    pub offset: UnitInterval,
     pub rgba: cssparser::RGBA,
     pub opacity: UnitInterval,
 }
@@ -187,21 +186,28 @@ impl GradientCommon {
         self.fallback = fallback.fallback.clone();
     }
 
-    fn add_color_stop(&mut self, mut offset: f64, rgba: cssparser::RGBA, opacity: UnitInterval) {
+    fn add_color_stop(
+        &mut self,
+        offset: UnitInterval,
+        rgba: cssparser::RGBA,
+        opacity: UnitInterval,
+    ) {
         if self.stops.is_none() {
             self.stops = Some(Vec::<ColorStop>::new());
         }
 
         if let Some(ref mut stops) = self.stops {
-            let last_offset: f64 = if !stops.is_empty() {
+            let last_offset = if !stops.is_empty() {
                 stops[stops.len() - 1].offset
             } else {
-                0.0
+                UnitInterval(0.0)
             };
 
-            if last_offset > offset {
-                offset = last_offset;
-            }
+            let offset = if offset > last_offset {
+                offset
+            } else {
+                last_offset
+            };
 
             stops.push(ColorStop {
                 offset,
@@ -391,12 +397,18 @@ impl Gradient {
                         StopColor(cssparser::Color::CurrentColor) => values.color.0,
                         StopColor(cssparser::Color::RGBA(ref rgba)) => *rgba,
                     };
+
                     self.add_color_stop(stop.get_offset(), rgba, values.stop_opacity.0);
                 })
             });
     }
 
-    fn add_color_stop(&mut self, offset: f64, rgba: cssparser::RGBA, opacity: UnitInterval) {
+    fn add_color_stop(
+        &mut self,
+        offset: UnitInterval,
+        rgba: cssparser::RGBA,
+        opacity: UnitInterval,
+    ) {
         self.common.add_color_stop(offset, rgba, opacity);
     }
 
@@ -407,11 +419,12 @@ impl Gradient {
     ) {
         if let Some(stops) = self.common.stops.as_ref() {
             for stop in stops {
+                let UnitInterval(stop_offset) = stop.offset;
                 let &UnitInterval(o) = opacity;
                 let UnitInterval(stop_opacity) = stop.opacity;
 
                 pattern.add_color_stop_rgba(
-                    stop.offset,
+                    stop_offset,
                     f64::from(stop.rgba.red_f32()),
                     f64::from(stop.rgba.green_f32()),
                     f64::from(stop.rgba.blue_f32()),
@@ -430,18 +443,20 @@ impl Gradient {
     }
 }
 
-fn acquire_gradient<'a>(draw_ctx: &'a mut DrawingCtx, name: &Fragment) -> Option<AcquiredNode> {
-    if let Some(acquired) = draw_ctx.get_acquired_node(name) {
-        let node_type = acquired.get().get_type();
+fn acquire_gradient<'a>(
+    draw_ctx: &'a mut DrawingCtx,
+    name: Option<&Fragment>,
+) -> Option<AcquiredNode> {
+    name.and_then(move |fragment| draw_ctx.get_acquired_node(fragment))
+        .and_then(|acquired| {
+            let node_type = acquired.get().get_type();
 
-        if node_type == NodeType::LinearGradient || node_type == NodeType::RadialGradient {
-            return Some(acquired);
-        }
-    }
-
-    rsvg_log!("element \"{}\" does not exist or is not a gradient", name);
-
-    None
+            if node_type == NodeType::LinearGradient || node_type == NodeType::RadialGradient {
+                Some(acquired)
+            } else {
+                None
+            }
+        })
 }
 
 fn set_common_on_pattern<P: cairo::PatternTrait + cairo::Gradient>(
@@ -515,49 +530,54 @@ fn fix_focus_point(fx: f64, fy: f64, cx: f64, cy: f64, radius: f64) -> (f64, f64
     (cx + dx, cy + dy)
 }
 
-impl PaintSource<Gradient> for NodeGradient {
+impl PaintSource for NodeGradient {
+    type Source = Gradient;
+
     fn resolve(
         &self,
         node: &RsvgNode,
         draw_ctx: &mut DrawingCtx,
         bbox: &BoundingBox,
-    ) -> Option<Gradient> {
-        let gradient =
-            node.with_impl(|i: &NodeGradient| i.get_gradient_with_color_stops_from_node(node));
+    ) -> Result<Option<Self::Source>, RenderingError> {
+        let node_gradient = node.get_impl::<NodeGradient>().unwrap();
+        let gradient = node_gradient.get_gradient_with_color_stops_from_node(node);
         let mut result = gradient.clone();
+        let mut stack = NodeStack::new();
 
         while !result.is_resolved() {
-            result
-                .common
-                .fallback
-                .as_ref()
-                .and_then(|fallback_name| acquire_gradient(draw_ctx, fallback_name))
-                .and_then(|acquired| {
-                    let fallback_node = acquired.get();
+            if let Some(acquired) = acquire_gradient(draw_ctx, result.common.fallback.as_ref()) {
+                let a_node = acquired.get();
 
-                    fallback_node.with_impl(|i: &NodeGradient| {
-                        let fallback_grad =
-                            i.get_gradient_with_color_stops_from_node(&fallback_node);
-                        result.resolve_from_fallback(&fallback_grad)
-                    });
-                    Some(())
-                })
-                .or_else(|| {
-                    result.resolve_from_defaults();
-                    Some(())
+                if stack.contains(a_node) {
+                    rsvg_log!(
+                        "circular reference in gradient {}",
+                        node.get_human_readable_name()
+                    );
+                    return Err(RenderingError::CircularReference);
+                }
+
+                a_node.with_impl(|i: &NodeGradient| {
+                    let fallback_grad = i.get_gradient_with_color_stops_from_node(&a_node);
+                    result.resolve_from_fallback(&fallback_grad)
                 });
+
+                stack.push(a_node);
+                continue;
+            }
+
+            result.resolve_from_defaults();
         }
 
         if result.bounds_are_valid(bbox) {
-            Some(result)
+            Ok(Some(result))
         } else {
-            None
+            Ok(None)
         }
     }
 
     fn set_pattern_on_draw_context(
         &self,
-        gradient: &Gradient,
+        gradient: &Self::Source,
         values: &ComputedValues,
         draw_ctx: &mut DrawingCtx,
         opacity: &UnitInterval,
@@ -637,12 +657,7 @@ impl NodeGradient {
 }
 
 impl NodeTrait for NodeGradient {
-    fn set_atts(
-        &self,
-        node: &RsvgNode,
-        _: *const RsvgHandle,
-        pbag: &PropertyBag<'_>,
-    ) -> NodeResult {
+    fn set_atts(&self, node: &RsvgNode, pbag: &PropertyBag<'_>) -> NodeResult {
         let mut g = self.gradient.borrow_mut();
 
         let mut x1 = None;
@@ -659,18 +674,9 @@ impl NodeTrait for NodeGradient {
         for (attr, value) in pbag.iter() {
             match attr {
                 // Attributes common to linear and radial gradients
-                Attribute::GradientUnits => {
-                    g.common.units = Some(parse("gradientUnits", value, ())?)
-                }
-
-                Attribute::GradientTransform => {
-                    g.common.affine = Some(parse("gradientTransform", value, ())?)
-                }
-
-                Attribute::SpreadMethod => {
-                    g.common.spread = Some(parse("spreadMethod", value, ())?)
-                }
-
+                Attribute::GradientUnits => g.common.units = Some(attr.parse(value, ())?),
+                Attribute::GradientTransform => g.common.affine = Some(attr.parse(value, ())?),
+                Attribute::SpreadMethod => g.common.spread = Some(attr.parse(value, ())?),
                 Attribute::XlinkHref => {
                     g.common.fallback =
                         Some(Fragment::parse(value).attribute(Attribute::XlinkHref)?)
@@ -678,16 +684,16 @@ impl NodeTrait for NodeGradient {
 
                 // Attributes specific to each gradient type.  The defaults mandated by the spec
                 // are in GradientVariant::resolve_from_defaults()
-                Attribute::X1 => x1 = Some(parse("x1", value, LengthDir::Horizontal)?),
-                Attribute::Y1 => y1 = Some(parse("y1", value, LengthDir::Vertical)?),
-                Attribute::X2 => x2 = Some(parse("x2", value, LengthDir::Horizontal)?),
-                Attribute::Y2 => y2 = Some(parse("y2", value, LengthDir::Vertical)?),
+                Attribute::X1 => x1 = Some(attr.parse(value, LengthDir::Horizontal)?),
+                Attribute::Y1 => y1 = Some(attr.parse(value, LengthDir::Vertical)?),
+                Attribute::X2 => x2 = Some(attr.parse(value, LengthDir::Horizontal)?),
+                Attribute::Y2 => y2 = Some(attr.parse(value, LengthDir::Vertical)?),
 
-                Attribute::Cx => cx = Some(parse("cx", value, LengthDir::Horizontal)?),
-                Attribute::Cy => cy = Some(parse("cy", value, LengthDir::Vertical)?),
-                Attribute::R => r = Some(parse("r", value, LengthDir::Both)?),
-                Attribute::Fx => fx = Some(parse("fx", value, LengthDir::Horizontal)?),
-                Attribute::Fy => fy = Some(parse("fy", value, LengthDir::Vertical)?),
+                Attribute::Cx => cx = Some(attr.parse(value, LengthDir::Horizontal)?),
+                Attribute::Cy => cy = Some(attr.parse(value, LengthDir::Vertical)?),
+                Attribute::R => r = Some(attr.parse(value, LengthDir::Both)?),
+                Attribute::Fx => fx = Some(attr.parse(value, LengthDir::Horizontal)?),
+                Attribute::Fy => fy = Some(attr.parse(value, LengthDir::Vertical)?),
 
                 _ => (),
             }
