@@ -9,10 +9,11 @@ use cairo::{self, ImageSurface, Status};
 use cairo_sys;
 use gdk_pixbuf::{Colorspace, Pixbuf, PixbufLoader, PixbufLoaderExt};
 use gdk_pixbuf_sys;
-use gio::{File as GFile, FileExt};
+use gio::{self, FileExt};
 use gio_sys;
 use glib::translate::*;
 use glib_sys;
+use gobject_sys;
 use libc;
 use url::Url;
 
@@ -194,7 +195,7 @@ impl Handle {
         }
     }
 
-    fn set_base_gfile(&self, file: &GFile) {
+    fn set_base_gfile(&self, file: &gio::File) {
         if let Some(uri) = file.get_uri() {
             self.set_base_url(&uri);
         } else {
@@ -205,8 +206,8 @@ impl Handle {
     pub fn read_stream_sync(
         &mut self,
         handle: *mut RsvgHandle,
-        stream: gio::InputStream,
-        cancellable: Option<gio::Cancellable>,
+        stream: &gio::InputStream,
+        cancellable: Option<&gio::Cancellable>,
     ) -> Result<(), LoadingError> {
         self.load_state.set(LoadState::Loading);
 
@@ -537,6 +538,20 @@ impl Handle {
 
         Ok(pixbuf)
     }
+
+    fn construct_read_stream_sync(
+        &mut self,
+        handle: *mut RsvgHandle,
+        stream: &gio::InputStream,
+        base_file: Option<&gio::File>,
+        cancellable: Option<&gio::Cancellable>,
+    ) -> Result<(), LoadingError> {
+        if let Some(file) = base_file {
+            self.set_base_gfile(file);
+        }
+
+        self.read_stream_sync(handle, stream, cancellable)
+    }
 }
 
 // Keep these in sync with rsvg.h:RsvgHandleFlags
@@ -574,6 +589,8 @@ impl LoadFlags {
 
 #[allow(improper_ctypes)]
 extern "C" {
+    fn rsvg_handle_new_with_flags(flags: u32) -> *mut RsvgHandle;
+
     fn rsvg_handle_new_from_gfile_sync(
         file: *const gio_sys::GFile,
         flags: u32,
@@ -598,7 +615,7 @@ pub fn lookup_fragment_id(handle: *const RsvgHandle, id: &str) -> Option<Rc<Node
 
 pub fn load_extern(load_options: &LoadOptions, aurl: &AllowedUrl) -> Result<*const RsvgHandle, ()> {
     unsafe {
-        let file = GFile::new_for_uri(aurl.url().as_str());
+        let file = gio::File::new_for_uri(aurl.url().as_str());
 
         let res = rsvg_handle_new_from_gfile_sync(
             file.to_glib_none().0,
@@ -755,7 +772,7 @@ pub unsafe extern "C" fn rsvg_handle_rust_get_base_gfile(
 
     match *handle.base_url.borrow() {
         None => ptr::null_mut(),
-        Some(ref url) => GFile::new_for_uri(url.as_str()).to_glib_full(),
+        Some(ref url) => gio::File::new_for_uri(url.as_str()).to_glib_full(),
     }
 }
 
@@ -768,7 +785,7 @@ pub unsafe extern "C" fn rsvg_handle_rust_set_base_gfile(
 
     assert!(!raw_gfile.is_null());
 
-    let file: GFile = from_glib_none(raw_gfile);
+    let file: gio::File = from_glib_none(raw_gfile);
 
     handle.set_base_gfile(&file);
 }
@@ -894,9 +911,9 @@ pub unsafe extern "C" fn rsvg_handle_rust_read_stream_sync(
     }
 
     let stream = from_glib_none(stream);
-    let cancellable = from_glib_none(cancellable);
+    let cancellable: Option<gio::Cancellable> = from_glib_none(cancellable);
 
-    match rhandle.read_stream_sync(handle, stream, cancellable) {
+    match rhandle.read_stream_sync(handle, &stream, cancellable.as_ref()) {
         Ok(()) => true.to_glib(),
 
         Err(e) => {
@@ -1156,3 +1173,35 @@ pub unsafe extern "C" fn rsvg_handle_rust_get_position_sub(
 
     res
 }
+
+#[no_mangle]
+pub unsafe extern "C" fn rsvg_handle_rust_new_from_stream_sync(
+    input_stream: *mut gio_sys::GInputStream,
+    base_file: *mut gio_sys::GFile,
+    flags: u32,
+    cancellable: *mut gio_sys::GCancellable,
+    error: *mut *mut glib_sys::GError,
+) -> *mut RsvgHandle {
+    let raw_handle = rsvg_handle_new_with_flags(flags);
+
+    let rhandle = get_rust_handle(raw_handle);
+
+    let base_file: Option<gio::File> = from_glib_none(base_file);
+    let stream = from_glib_none(input_stream);
+    let cancellable: Option<gio::Cancellable> = from_glib_none(cancellable);
+
+    match rhandle.construct_read_stream_sync(
+        raw_handle,
+        &stream,
+        base_file.as_ref(),
+        cancellable.as_ref(),
+    ) {
+        Ok(()) => raw_handle,
+
+        Err(e) => {
+            set_gerror(error, 0, &format!("{}", e));
+            gobject_sys::g_object_unref(raw_handle as *mut _);
+            ptr::null_mut()
+        }
+    }
+}
