@@ -2,7 +2,6 @@ use cairo;
 use cairo::{MatrixTrait, PatternTrait};
 use std::cell::{Cell, RefCell};
 
-use allowed_url::AllowedUrl;
 use aspect_ratio::AspectRatio;
 use attributes::Attribute;
 use bbox::BoundingBox;
@@ -10,10 +9,10 @@ use defs::Href;
 use drawing_ctx::DrawingCtx;
 use error::{NodeError, RenderingError};
 use float_eq_cairo::ApproxEqCairo;
-use handle::{self, RsvgHandle};
+use handle::{self, LoadOptions};
 use length::*;
 use node::*;
-use parsers::{parse, parse_and_validate};
+use parsers::ParseValue;
 use property_bag::PropertyBag;
 
 pub struct NodeImage {
@@ -22,6 +21,7 @@ pub struct NodeImage {
     y: Cell<Length>,
     w: Cell<Length>,
     h: Cell<Length>,
+    href: RefCell<Option<Href>>,
     surface: RefCell<Option<cairo::ImageSurface>>,
 }
 
@@ -33,63 +33,42 @@ impl NodeImage {
             y: Cell::new(Length::default()),
             w: Cell::new(Length::default()),
             h: Cell::new(Length::default()),
+            href: RefCell::new(None),
             surface: RefCell::new(None),
         }
     }
 }
 
 impl NodeTrait for NodeImage {
-    fn set_atts(
-        &self,
-        node: &RsvgNode,
-        handle: *const RsvgHandle,
-        pbag: &PropertyBag<'_>,
-    ) -> NodeResult {
+    fn set_atts(&self, node: &RsvgNode, pbag: &PropertyBag<'_>) -> NodeResult {
         // SVG element has overflow:hidden
         // https://www.w3.org/TR/SVG/styling.html#UAStyleSheet
         node.set_overflow_hidden();
 
         for (attr, value) in pbag.iter() {
             match attr {
-                Attribute::X => self.x.set(parse("x", value, LengthDir::Horizontal)?),
-                Attribute::Y => self.y.set(parse("y", value, LengthDir::Vertical)?),
-                Attribute::Width => self.w.set(parse_and_validate(
-                    "width",
+                Attribute::X => self.x.set(attr.parse(value, LengthDir::Horizontal)?),
+                Attribute::Y => self.y.set(attr.parse(value, LengthDir::Vertical)?),
+                Attribute::Width => self.w.set(attr.parse_and_validate(
                     value,
                     LengthDir::Horizontal,
                     Length::check_nonnegative,
                 )?),
-                Attribute::Height => self.h.set(parse_and_validate(
-                    "height",
+                Attribute::Height => self.h.set(attr.parse_and_validate(
                     value,
                     LengthDir::Vertical,
                     Length::check_nonnegative,
                 )?),
 
-                Attribute::PreserveAspectRatio => {
-                    self.aspect.set(parse("preserveAspectRatio", value, ())?)
-                }
+                Attribute::PreserveAspectRatio => self.aspect.set(attr.parse(value, ())?),
 
                 // "path" is used by some older Adobe Illustrator versions
                 Attribute::XlinkHref | Attribute::Path => {
                     // FIXME: use better errors here; these should be loading errors
-
-                    let href = Href::without_fragment(value)
-                        .map_err(|_| NodeError::value_error(attr, "fragment not allowed here"))?;
-
-                    let url = match href {
-                        Href::PlainUri(u) => u,
-                        _ => unreachable!(),
-                    };
-
-                    let aurl = AllowedUrl::from_href(&url, handle::get_base_url(handle).as_ref())
-                        .map_err(|_| NodeError::value_error(attr, "invalid URL"))?;
-
-                    *self.surface.borrow_mut() = Some(
-                        // FIXME: translate the error better here
-                        handle::load_image_to_surface(handle as *mut _, &aurl)
-                            .map_err(|_| NodeError::value_error(attr, "could not load image"))?,
-                    );
+                    *self.href.borrow_mut() =
+                        Some(Href::without_fragment(value).map_err(|_| {
+                            NodeError::value_error(attr, "fragment not allowed here")
+                        })?);
                 }
 
                 _ => (),
@@ -99,6 +78,26 @@ impl NodeTrait for NodeImage {
         Ok(())
     }
 
+    fn resolve_resources(&self, load_options: &LoadOptions) -> NodeResult {
+        match *self.href.borrow() {
+            None => (),
+            Some(Href::PlainUri(ref url)) => {
+                *self.surface.borrow_mut() = Some(
+                    // FIXME: translate the error better here
+                    handle::load_image_to_surface(load_options, &url).map_err(|e| {
+                        NodeError::value_error(
+                            Attribute::XlinkHref,
+                            &format!("could not load image: {}", e),
+                        )
+                    })?,
+                );
+            }
+            _ => unreachable!(),
+        };
+
+        Ok(())
+    }
+
     fn draw(
         &self,
         node: &RsvgNode,
@@ -127,6 +126,12 @@ impl NodeTrait for NodeImage {
                     dc.clip(x, y, w, h);
                 }
 
+                let width = surface.get_width();
+                let height = surface.get_height();
+                if clipping || width == 0 || height == 0 {
+                    return Ok(());
+                }
+
                 // The bounding box for <image> is decided by the values of x, y, w, h and not by
                 // the final computed image bounds.
                 let bbox = BoundingBox::new(&dc.get_cairo_context().get_matrix()).with_rect(Some(
@@ -138,12 +143,6 @@ impl NodeTrait for NodeImage {
                     },
                 ));
 
-                let width = surface.get_width();
-                let height = surface.get_height();
-                if clipping || width == 0 || height == 0 {
-                    return Ok(());
-                }
-
                 let width = f64::from(width);
                 let height = f64::from(height);
 
