@@ -12,12 +12,10 @@ use defs::Fragment;
 use drawing_ctx::DrawingCtx;
 use error::*;
 use float_eq_cairo::ApproxEqCairo;
-use handle::RsvgHandle;
 use iri::IRI;
 use length::{Length, LengthDir};
 use node::*;
-use parsers::ParseError;
-use parsers::{parse, parse_and_validate, Parse};
+use parsers::{Parse, ParseError, ParseValue};
 use path_builder::*;
 use property_bag::PropertyBag;
 use state::{ComputedValues, SpecifiedValue, State};
@@ -208,45 +206,36 @@ impl NodeMarker {
 }
 
 impl NodeTrait for NodeMarker {
-    fn set_atts(
-        &self,
-        node: &RsvgNode,
-        _: *const RsvgHandle,
-        pbag: &PropertyBag<'_>,
-    ) -> NodeResult {
+    fn set_atts(&self, node: &RsvgNode, pbag: &PropertyBag<'_>) -> NodeResult {
         // marker element has overflow:hidden
         // https://www.w3.org/TR/SVG/styling.html#UAStyleSheet
         node.set_overflow_hidden();
 
         for (attr, value) in pbag.iter() {
             match attr {
-                Attribute::MarkerUnits => self.units.set(parse("markerUnits", value, ())?),
+                Attribute::MarkerUnits => self.units.set(attr.parse(value, ())?),
 
-                Attribute::RefX => self.ref_x.set(parse("refX", value, LengthDir::Horizontal)?),
+                Attribute::RefX => self.ref_x.set(attr.parse(value, LengthDir::Horizontal)?),
 
-                Attribute::RefY => self.ref_y.set(parse("refY", value, LengthDir::Vertical)?),
+                Attribute::RefY => self.ref_y.set(attr.parse(value, LengthDir::Vertical)?),
 
-                Attribute::MarkerWidth => self.width.set(parse_and_validate(
-                    "markerWidth",
+                Attribute::MarkerWidth => self.width.set(attr.parse_and_validate(
                     value,
                     LengthDir::Horizontal,
                     Length::check_nonnegative,
                 )?),
 
-                Attribute::MarkerHeight => self.height.set(parse_and_validate(
-                    "markerHeight",
+                Attribute::MarkerHeight => self.height.set(attr.parse_and_validate(
                     value,
                     LengthDir::Vertical,
                     Length::check_nonnegative,
                 )?),
 
-                Attribute::Orient => self.orient.set(parse("orient", value, ())?),
+                Attribute::Orient => self.orient.set(attr.parse(value, ())?),
 
-                Attribute::PreserveAspectRatio => {
-                    self.aspect.set(parse("preserveAspectRatio", value, ())?)
-                }
+                Attribute::PreserveAspectRatio => self.aspect.set(attr.parse(value, ())?),
 
-                Attribute::ViewBox => self.vbox.set(Some(parse("viewBox", value, ())?)),
+                Attribute::ViewBox => self.vbox.set(Some(attr.parse(value, ())?)),
 
                 _ => (),
             }
