@@ -8,7 +8,7 @@ use cond::{locale_from_environment, RequiredExtensions, RequiredFeatures, System
 use css::CssStyles;
 use drawing_ctx::DrawingCtx;
 use error::*;
-use handle::RsvgHandle;
+use handle::LoadOptions;
 use parsers::Parse;
 use property_bag::PropertyBag;
 use state::{ComputedValues, Overflow, SpecifiedValue, State};
@@ -97,17 +97,18 @@ pub trait NodeTrait: Downcast {
     /// Sets per-node attributes from the `pbag`
     ///
     /// Each node is supposed to iterate the `pbag`, and parse any attributes it needs.
-    fn set_atts(
-        &self,
-        node: &RsvgNode,
-        handle: *const RsvgHandle,
-        pbag: &PropertyBag<'_>,
-    ) -> NodeResult;
+    fn set_atts(&self, node: &RsvgNode, pbag: &PropertyBag<'_>) -> NodeResult;
 
     /// Sets any special-cased properties that the node may have, that are different
     /// from defaults in the node's `State`.
     fn set_overridden_properties(&self, _state: &mut State) {}
 
+    /// Load resources specified in the attributes.
+    fn resolve_resources(&self, _load_options: &LoadOptions) -> NodeResult {
+        // Most of the nodes do not need to load resources
+        Ok(())
+    }
+
     fn draw(
         &self,
         _node: &RsvgNode,
@@ -348,7 +349,7 @@ impl Node {
         self.data.cond.get()
     }
 
-    pub fn set_atts(&self, node: &RsvgNode, handle: *const RsvgHandle, pbag: &PropertyBag<'_>) {
+    pub fn set_atts(&self, node: &RsvgNode, pbag: &PropertyBag<'_>) {
         for (attr, value) in pbag.iter() {
             match attr {
                 Attribute::Transform => match Matrix::parse_str(value, ()) {
@@ -371,7 +372,7 @@ impl Node {
             }
         }
 
-        match self.data.node_impl.set_atts(node, handle, pbag) {
+        match self.data.node_impl.set_atts(node, pbag) {
             Ok(_) => (),
             Err(e) => {
                 self.set_error(e);
@@ -444,6 +445,15 @@ impl Node {
         }
     }
 
+    pub fn resolve_resources(&self, load_options: &LoadOptions) {
+        match self.data.node_impl.resolve_resources(load_options) {
+            Ok(_) => (),
+            Err(e) => {
+                self.set_error(e);
+            }
+        }
+    }
+
     /// Implements a very limited CSS selection engine
     fn set_css_styles(&self, css_styles: &CssStyles) {
         // Try to properly support all of the following, including inheritance:
