@@ -1,15 +1,16 @@
 use cairo;
 use std::cell::Cell;
 use std::cell::RefCell;
+use std::ops::Deref;
 
 use attributes::Attribute;
+use cssparser::{Parser, Token};
 use drawing_ctx::DrawingCtx;
 use error::*;
-use handle::RsvgHandle;
 use length::*;
 use marker;
 use node::*;
-use parsers::{self, parse, parse_and_validate};
+use parsers::{CssParserExt, Parse, ParseValue};
 use path_builder::*;
 use path_parser;
 use property_bag::PropertyBag;
@@ -123,12 +124,7 @@ impl NodePath {
 }
 
 impl NodeTrait for NodePath {
-    fn set_atts(
-        &self,
-        node: &RsvgNode,
-        _: *const RsvgHandle,
-        pbag: &PropertyBag<'_>,
-    ) -> NodeResult {
+    fn set_atts(&self, node: &RsvgNode, pbag: &PropertyBag<'_>) -> NodeResult {
         for (attr, value) in pbag.iter() {
             if attr == Attribute::D {
                 let mut builder = PathBuilder::new();
@@ -174,8 +170,49 @@ enum PolyKind {
     Closed,
 }
 
+#[derive(Debug, PartialEq)]
+struct Points(Vec<(f64, f64)>);
+
+impl Deref for Points {
+    type Target = [(f64, f64)];
+
+    fn deref(&self) -> &[(f64, f64)] {
+        &self.0
+    }
+}
+
+// Parse a list-of-points as for polyline and polygon elements
+// https://www.w3.org/TR/SVG/shapes.html#PointsBNF
+impl Parse for Points {
+    type Data = ();
+    type Err = ValueErrorKind;
+
+    fn parse(parser: &mut Parser<'_, '_>, _: ()) -> Result<Points, ValueErrorKind> {
+        let mut v = Vec::new();
+
+        loop {
+            let x = f64::from(parser.expect_finite_number()?);
+            parser.optional_comma();
+            let y = f64::from(parser.expect_finite_number()?);
+
+            v.push((x, y));
+
+            if parser.is_exhausted() {
+                break;
+            }
+
+            match parser.next_including_whitespace() {
+                Ok(&Token::WhiteSpace(_)) => (),
+                _ => parser.optional_comma(),
+            }
+        }
+
+        Ok(Points(v))
+    }
+}
+
 pub struct NodePoly {
-    points: RefCell<Option<Vec<(f64, f64)>>>,
+    points: RefCell<Option<Points>>,
     kind: PolyKind,
 }
 
@@ -196,22 +233,11 @@ impl NodePoly {
 }
 
 impl NodeTrait for NodePoly {
-    fn set_atts(&self, _: &RsvgNode, _: *const RsvgHandle, pbag: &PropertyBag<'_>) -> NodeResult {
+    fn set_atts(&self, _: &RsvgNode, pbag: &PropertyBag<'_>) -> NodeResult {
         for (attr, value) in pbag.iter() {
             // support for svg < 1.0 which used verts
             if attr == Attribute::Points || attr == Attribute::Verts {
-                let result = parsers::list_of_points(value.trim());
-
-                match result {
-                    Ok(v) => {
-                        *self.points.borrow_mut() = Some(v);
-                        break;
-                    }
-
-                    Err(e) => {
-                        return Err(NodeError::attribute_error(attr, e));
-                    }
-                }
+                *self.points.borrow_mut() = attr.parse(value.trim(), ()).map(Some)?;
             }
         }
 
@@ -268,13 +294,13 @@ impl NodeLine {
 }
 
 impl NodeTrait for NodeLine {
-    fn set_atts(&self, _: &RsvgNode, _: *const RsvgHandle, pbag: &PropertyBag<'_>) -> NodeResult {
+    fn set_atts(&self, _: &RsvgNode, pbag: &PropertyBag<'_>) -> NodeResult {
         for (attr, value) in pbag.iter() {
             match attr {
-                Attribute::X1 => self.x1.set(parse("x1", value, LengthDir::Horizontal)?),
-                Attribute::Y1 => self.y1.set(parse("y1", value, LengthDir::Vertical)?),
-                Attribute::X2 => self.x2.set(parse("x2", value, LengthDir::Horizontal)?),
-                Attribute::Y2 => self.y2.set(parse("y2", value, LengthDir::Vertical)?),
+                Attribute::X1 => self.x1.set(attr.parse(value, LengthDir::Horizontal)?),
+                Attribute::Y1 => self.y1.set(attr.parse(value, LengthDir::Vertical)?),
+                Attribute::X2 => self.x2.set(attr.parse(value, LengthDir::Horizontal)?),
+                Attribute::Y2 => self.y2.set(attr.parse(value, LengthDir::Vertical)?),
                 _ => (),
             }
         }
@@ -334,27 +360,23 @@ impl NodeRect {
 }
 
 impl NodeTrait for NodeRect {
-    fn set_atts(&self, _: &RsvgNode, _: *const RsvgHandle, pbag: &PropertyBag<'_>) -> NodeResult {
+    fn set_atts(&self, _: &RsvgNode, pbag: &PropertyBag<'_>) -> NodeResult {
         for (attr, value) in pbag.iter() {
             match attr {
-                Attribute::X => self.x.set(parse("x", value, LengthDir::Horizontal)?),
-                Attribute::Y => self.y.set(parse("y", value, LengthDir::Vertical)?),
-                Attribute::Width => self.w.set(parse_and_validate(
-                    "width",
+                Attribute::X => self.x.set(attr.parse(value, LengthDir::Horizontal)?),
+                Attribute::Y => self.y.set(attr.parse(value, LengthDir::Vertical)?),
+                Attribute::Width => self.w.set(attr.parse_and_validate(
                     value,
                     LengthDir::Horizontal,
                     Length::check_nonnegative,
                 )?),
-                Attribute::Height => self.h.set(parse_and_validate(
-                    "height",
+                Attribute::Height => self.h.set(attr.parse_and_validate(
                     value,
                     LengthDir::Vertical,
                     Length::check_nonnegative,
                 )?),
-
                 Attribute::Rx => self.rx.set(
-                    parse_and_validate(
-                        "rx",
+                    attr.parse_and_validate(
                         value,
                         LengthDir::Horizontal,
                         Length::check_nonnegative,
@@ -362,7 +384,7 @@ impl NodeTrait for NodeRect {
                     .map(Some)?,
                 ),
                 Attribute::Ry => self.ry.set(
-                    parse_and_validate("ry", value, LengthDir::Vertical, Length::check_nonnegative)
+                    attr.parse_and_validate(value, LengthDir::Vertical, Length::check_nonnegative)
                         .map(Some)?,
                 ),
 
@@ -538,13 +560,12 @@ impl NodeCircle {
 }
 
 impl NodeTrait for NodeCircle {
-    fn set_atts(&self, _: &RsvgNode, _: *const RsvgHandle, pbag: &PropertyBag<'_>) -> NodeResult {
+    fn set_atts(&self, _: &RsvgNode, pbag: &PropertyBag<'_>) -> NodeResult {
         for (attr, value) in pbag.iter() {
             match attr {
-                Attribute::Cx => self.cx.set(parse("cx", value, LengthDir::Horizontal)?),
-                Attribute::Cy => self.cy.set(parse("cy", value, LengthDir::Vertical)?),
-                Attribute::R => self.r.set(parse_and_validate(
-                    "r",
+                Attribute::Cx => self.cx.set(attr.parse(value, LengthDir::Horizontal)?),
+                Attribute::Cy => self.cy.set(attr.parse(value, LengthDir::Vertical)?),
+                Attribute::R => self.r.set(attr.parse_and_validate(
                     value,
                     LengthDir::Both,
                     Length::check_nonnegative,
@@ -595,20 +616,17 @@ impl NodeEllipse {
 }
 
 impl NodeTrait for NodeEllipse {
-    fn set_atts(&self, _: &RsvgNode, _: *const RsvgHandle, pbag: &PropertyBag<'_>) -> NodeResult {
+    fn set_atts(&self, _: &RsvgNode, pbag: &PropertyBag<'_>) -> NodeResult {
         for (attr, value) in pbag.iter() {
             match attr {
-                Attribute::Cx => self.cx.set(parse("cx", value, LengthDir::Horizontal)?),
-                Attribute::Cy => self.cy.set(parse("cy", value, LengthDir::Vertical)?),
-
-                Attribute::Rx => self.rx.set(parse_and_validate(
-                    "rx",
+                Attribute::Cx => self.cx.set(attr.parse(value, LengthDir::Horizontal)?),
+                Attribute::Cy => self.cy.set(attr.parse(value, LengthDir::Vertical)?),
+                Attribute::Rx => self.rx.set(attr.parse_and_validate(
                     value,
                     LengthDir::Horizontal,
                     Length::check_nonnegative,
                 )?),
-                Attribute::Ry => self.ry.set(parse_and_validate(
-                    "ry",
+                Attribute::Ry => self.ry.set(attr.parse_and_validate(
                     value,
                     LengthDir::Vertical,
                     Length::check_nonnegative,
@@ -640,3 +658,39 @@ impl NodeTrait for NodeEllipse {
         render_ellipse(cx, cy, rx, ry, draw_ctx, node, values, clipping)
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    #[test]
+    fn parses_points() {
+        assert_eq!(Points::parse_str(" 1 2 ", ()), Ok(Points(vec![(1.0, 2.0)])));
+        assert_eq!(
+            Points::parse_str("1 2 3 4", ()),
+            Ok(Points(vec![(1.0, 2.0), (3.0, 4.0)]))
+        );
+        assert_eq!(
+            Points::parse_str("1,2,3,4", ()),
+            Ok(Points(vec![(1.0, 2.0), (3.0, 4.0)]))
+        );
+        assert_eq!(
+            Points::parse_str("1,2 3,4", ()),
+            Ok(Points(vec![(1.0, 2.0), (3.0, 4.0)]))
+        );
+        assert_eq!(
+            Points::parse_str("1,2 -3,4", ()),
+            Ok(Points(vec![(1.0, 2.0), (-3.0, 4.0)]))
+        );
+        assert_eq!(
+            Points::parse_str("1,2,-3,4", ()),
+            Ok(Points(vec![(1.0, 2.0), (-3.0, 4.0)]))
+        );
+    }
+
+    #[test]
+    fn errors_on_invalid_points() {
+        assert!(Points::parse_str("-1-2-3-4", ()).is_err());
+        assert!(Points::parse_str("1 2-3,-4", ()).is_err());
+    }
+}
