@@ -12,7 +12,7 @@ use paint_server::PaintServer;
 use parsers::{Parse, ParseError};
 use property_bag::PropertyBag;
 use property_macros::Property;
-use unitinterval::UnitInterval;
+use unit_interval::UnitInterval;
 
 /// Representation of a single CSS property value.
 ///
@@ -204,10 +204,6 @@ impl ComputedValues {
             _ => false,
         }
     }
-
-    pub fn text_gravity_is_vertical(&self) -> bool {
-        self.writing_mode.is_vertical()
-    }
 }
 
 macro_rules! compute_value {
