@@ -9,10 +9,9 @@ use dpi::Dpi;
 use drawing_ctx::DrawingCtx;
 use error::{AttributeResultExt, RenderingError};
 use float_eq_cairo::ApproxEqCairo;
-use handle::RsvgHandle;
 use length::*;
 use node::*;
-use parsers::{parse, parse_and_validate, Parse};
+use parsers::{Parse, ParseValue};
 use property_bag::{OwnedPropertyBag, PropertyBag};
 use state::Overflow;
 use viewbox::*;
@@ -27,7 +26,7 @@ impl NodeGroup {
 }
 
 impl NodeTrait for NodeGroup {
-    fn set_atts(&self, _: &RsvgNode, _: *const RsvgHandle, _: &PropertyBag<'_>) -> NodeResult {
+    fn set_atts(&self, _: &RsvgNode, _: &PropertyBag<'_>) -> NodeResult {
         Ok(())
     }
 
@@ -55,7 +54,7 @@ impl NodeDefs {
 }
 
 impl NodeTrait for NodeDefs {
-    fn set_atts(&self, _: &RsvgNode, _: *const RsvgHandle, _: &PropertyBag<'_>) -> NodeResult {
+    fn set_atts(&self, _: &RsvgNode, _: &PropertyBag<'_>) -> NodeResult {
         Ok(())
     }
 }
@@ -69,7 +68,7 @@ impl NodeSwitch {
 }
 
 impl NodeTrait for NodeSwitch {
-    fn set_atts(&self, _: &RsvgNode, _: *const RsvgHandle, _: &PropertyBag<'_>) -> NodeResult {
+    fn set_atts(&self, _: &RsvgNode, _: &PropertyBag<'_>) -> NodeResult {
         Ok(())
     }
 
@@ -147,12 +146,7 @@ impl NodeSvg {
 }
 
 impl NodeTrait for NodeSvg {
-    fn set_atts(
-        &self,
-        node: &RsvgNode,
-        _: *const RsvgHandle,
-        pbag: &PropertyBag<'_>,
-    ) -> NodeResult {
+    fn set_atts(&self, node: &RsvgNode, pbag: &PropertyBag<'_>) -> NodeResult {
         // SVG element has overflow:hidden
         // https://www.w3.org/TR/SVG/styling.html#UAStyleSheet
         node.set_overflow_hidden();
@@ -164,37 +158,34 @@ impl NodeTrait for NodeSvg {
         for (attr, value) in pbag.iter() {
             match attr {
                 Attribute::PreserveAspectRatio => {
-                    self.preserve_aspect_ratio
-                        .set(parse("preserveAspectRatio", value, ())?)
+                    self.preserve_aspect_ratio.set(attr.parse(value, ())?)
                 }
 
                 Attribute::X => {
                     if is_inner_svg {
-                        self.x.set(parse("x", value, LengthDir::Horizontal)?);
+                        self.x.set(attr.parse(value, LengthDir::Horizontal)?);
                     }
                 }
 
                 Attribute::Y => {
                     if is_inner_svg {
-                        self.y.set(parse("y", value, LengthDir::Vertical)?);
+                        self.y.set(attr.parse(value, LengthDir::Vertical)?);
                     }
                 }
 
-                Attribute::Width => self.w.set(parse_and_validate(
-                    "width",
+                Attribute::Width => self.w.set(attr.parse_and_validate(
                     value,
                     LengthDir::Horizontal,
                     Length::check_nonnegative,
                 )?),
 
-                Attribute::Height => self.h.set(parse_and_validate(
-                    "height",
+                Attribute::Height => self.h.set(attr.parse_and_validate(
                     value,
                     LengthDir::Vertical,
                     Length::check_nonnegative,
                 )?),
 
-                Attribute::ViewBox => self.vbox.set(parse("viewBox", value, ()).map(Some)?),
+                Attribute::ViewBox => self.vbox.set(attr.parse(value, ()).map(Some)?),
 
                 _ => (),
             }
@@ -268,7 +259,7 @@ impl NodeUse {
 }
 
 impl NodeTrait for NodeUse {
-    fn set_atts(&self, _: &RsvgNode, _: *const RsvgHandle, pbag: &PropertyBag<'_>) -> NodeResult {
+    fn set_atts(&self, _: &RsvgNode, pbag: &PropertyBag<'_>) -> NodeResult {
         for (attr, value) in pbag.iter() {
             match attr {
                 Attribute::XlinkHref => {
@@ -276,12 +267,11 @@ impl NodeTrait for NodeUse {
                         Some(Fragment::parse(value).attribute(Attribute::XlinkHref)?)
                 }
 
-                Attribute::X => self.x.set(parse("x", value, LengthDir::Horizontal)?),
-                Attribute::Y => self.y.set(parse("y", value, LengthDir::Vertical)?),
+                Attribute::X => self.x.set(attr.parse(value, LengthDir::Horizontal)?),
+                Attribute::Y => self.y.set(attr.parse(value, LengthDir::Vertical)?),
 
                 Attribute::Width => self.w.set(
-                    parse_and_validate(
-                        "width",
+                    attr.parse_and_validate(
                         value,
                         LengthDir::Horizontal,
                         Length::check_nonnegative,
@@ -289,13 +279,8 @@ impl NodeTrait for NodeUse {
                     .map(Some)?,
                 ),
                 Attribute::Height => self.h.set(
-                    parse_and_validate(
-                        "height",
-                        value,
-                        LengthDir::Vertical,
-                        Length::check_nonnegative,
-                    )
-                    .map(Some)?,
+                    attr.parse_and_validate(value, LengthDir::Vertical, Length::check_nonnegative)
+                        .map(Some)?,
                 ),
 
                 _ => (),
@@ -429,12 +414,7 @@ impl NodeSymbol {
 }
 
 impl NodeTrait for NodeSymbol {
-    fn set_atts(
-        &self,
-        node: &RsvgNode,
-        _: *const RsvgHandle,
-        pbag: &PropertyBag<'_>,
-    ) -> NodeResult {
+    fn set_atts(&self, node: &RsvgNode, pbag: &PropertyBag<'_>) -> NodeResult {
         // symbol element has overflow:hidden
         // https://www.w3.org/TR/SVG/styling.html#UAStyleSheet
         node.set_overflow_hidden();
@@ -442,11 +422,10 @@ impl NodeTrait for NodeSymbol {
         for (attr, value) in pbag.iter() {
             match attr {
                 Attribute::PreserveAspectRatio => {
-                    self.preserve_aspect_ratio
-                        .set(parse("preserveAspectRatio", value, ())?)
+                    self.preserve_aspect_ratio.set(attr.parse(value, ())?)
                 }
 
-                Attribute::ViewBox => self.vbox.set(parse("viewBox", value, ()).map(Some)?),
+                Attribute::ViewBox => self.vbox.set(attr.parse(value, ()).map(Some)?),
 
                 _ => (),
             }
