@@ -1,15 +1,19 @@
-use pango::{self, ContextExt, LayoutExt};
+use glib::translate::*;
+use pango::{self, ContextExt, FontMapExt, LayoutExt};
+use pango_sys;
+use pangocairo;
 use std::cell::{Cell, RefCell};
 
 use attributes::Attribute;
+use bbox::BoundingBox;
 use defs::Fragment;
 use drawing_ctx::DrawingCtx;
 use error::{AttributeResultExt, RenderingError};
+use float_eq_cairo::ApproxEqCairo;
 use font_props::FontWeightSpec;
-use handle::RsvgHandle;
 use length::*;
 use node::{CascadedValues, NodeResult, NodeTrait, NodeType, RsvgNode};
-use parsers::parse;
+use parsers::ParseValue;
 use property_bag::PropertyBag;
 use space::{xml_space_normalize, NormalizeDefault, XmlSpaceNormalize};
 use state::{
@@ -19,8 +23,10 @@ use state::{
     FontStyle,
     FontVariant,
     TextAnchor,
+    TextRendering,
     UnicodeBidi,
     WritingMode,
+    XmlLang,
     XmlSpace,
 };
 
@@ -242,7 +248,7 @@ impl PositionedSpan {
             .map(|l| l.normalize(&values, &params))
             .unwrap_or(0.0);
 
-        let (render_x, render_y) = if values.text_gravity_is_vertical() {
+        let (render_x, render_y) = if values.writing_mode.is_vertical() {
             (x + offset + dx, y + dy)
         } else {
             (x + dx, y - offset + dy)
@@ -257,13 +263,143 @@ impl PositionedSpan {
     }
 
     fn draw(&self, draw_ctx: &mut DrawingCtx, clipping: bool) -> Result<(), RenderingError> {
-        draw_ctx.draw_pango_layout(
-            &self.layout,
-            &self.values,
-            self.rendered_position.0,
-            self.rendered_position.1,
-            clipping,
-        )
+        let cr = draw_ctx.get_cairo_context();
+        cr.save();
+
+        draw_ctx.set_affine_on_cr(&cr);
+
+        let affine = cr.get_matrix();
+
+        let gravity = self.layout.get_context().unwrap().get_gravity();
+        let bbox = self.compute_text_bbox(&affine, gravity);
+        if bbox.is_none() {
+            cr.restore();
+            return Ok(());
+        }
+
+        let bbox = bbox.unwrap();
+
+        if !clipping {
+            draw_ctx.insert_bbox(&bbox);
+        }
+
+        cr.set_antialias(cairo::Antialias::from(self.values.text_rendering));
+
+        draw_ctx.setup_cr_for_stroke(&cr, &self.values);
+
+        cr.move_to(self.rendered_position.0, self.rendered_position.1);
+
+        let rotation = unsafe { pango_sys::pango_gravity_to_rotation(gravity.to_glib()) };
+        if !rotation.approx_eq_cairo(&0.0) {
+            cr.rotate(-rotation);
+        }
+
+        let current_color = &self.values.color.0;
+
+        let fill_opacity = &self.values.fill_opacity.0;
+
+        let res = if !clipping {
+            draw_ctx
+                .set_source_paint_server(&self.values.fill.0, fill_opacity, &bbox, current_color)
+                .and_then(|had_paint_server| {
+                    if had_paint_server {
+                        pangocairo::functions::update_layout(&cr, &self.layout);
+                        pangocairo::functions::show_layout(&cr, &self.layout);
+                    };
+                    Ok(())
+                })
+        } else {
+            Ok(())
+        };
+
+        if res.is_ok() {
+            let stroke_opacity = &self.values.stroke_opacity.0;
+
+            let mut need_layout_path = clipping;
+
+            let res = if !clipping {
+                draw_ctx
+                    .set_source_paint_server(
+                        &self.values.stroke.0,
+                        stroke_opacity,
+                        &bbox,
+                        &current_color,
+                    )
+                    .and_then(|had_paint_server| {
+                        if had_paint_server {
+                            need_layout_path = true;
+                        }
+                        Ok(())
+                    })
+            } else {
+                Ok(())
+            };
+
+            if res.is_ok() {
+                if need_layout_path {
+                    pangocairo::functions::update_layout(&cr, &self.layout);
+                    pangocairo::functions::layout_path(&cr, &self.layout);
+
+                    if !clipping {
+                        let ib = BoundingBox::new(&affine).with_ink_extents(cr.stroke_extents());
+                        cr.stroke();
+                        draw_ctx.insert_bbox(&ib);
+                    }
+                }
+            }
+        }
+
+        cr.restore();
+
+        res
+    }
+
+    fn compute_text_bbox(
+        &self,
+        affine: &cairo::Matrix,
+        gravity: pango::Gravity,
+    ) -> Option<BoundingBox> {
+        let (ink, _) = self.layout.get_extents();
+        if ink.width == 0 || ink.height == 0 {
+            return None;
+        }
+
+        let (x, y) = self.rendered_position;
+        let ink_x = f64::from(ink.x);
+        let ink_y = f64::from(ink.y);
+        let ink_width = f64::from(ink.width);
+        let ink_height = f64::from(ink.height);
+        let pango_scale = f64::from(pango::SCALE);
+
+        let rect = if gravity_is_vertical(gravity) {
+            cairo::Rectangle {
+                x: x + (ink_x - ink_height) / pango_scale,
+                y: y + ink_y / pango_scale,
+                width: ink_height / pango_scale,
+                height: ink_width / pango_scale,
+            }
+        } else {
+            cairo::Rectangle {
+                x: x + ink_x / pango_scale,
+                y: y + ink_y / pango_scale,
+                width: ink_width / pango_scale,
+                height: ink_height / pango_scale,
+            }
+        };
+
+        let bbox = BoundingBox::new(affine)
+            .with_rect(Some(rect))
+            .with_ink_rect(Some(rect));
+
+        Some(bbox)
+    }
+}
+
+// FIXME: should the pango crate provide this like PANGO_GRAVITY_IS_VERTICAL() ?
+fn gravity_is_vertical(gravity: pango::Gravity) -> bool {
+    match gravity {
+        pango::Gravity::East | pango::Gravity::West => true,
+        _ => false,
     }
 }
 
@@ -405,7 +541,7 @@ impl NodeChars {
 }
 
 impl NodeTrait for NodeChars {
-    fn set_atts(&self, _: &RsvgNode, _: *const RsvgHandle, _: &PropertyBag<'_>) -> NodeResult {
+    fn set_atts(&self, _: &RsvgNode, _: &PropertyBag<'_>) -> NodeResult {
         Ok(())
     }
 }
@@ -448,17 +584,17 @@ impl NodeText {
 }
 
 impl NodeTrait for NodeText {
-    fn set_atts(&self, _: &RsvgNode, _: *const RsvgHandle, pbag: &PropertyBag<'_>) -> NodeResult {
+    fn set_atts(&self, _: &RsvgNode, pbag: &PropertyBag<'_>) -> NodeResult {
         for (attr, value) in pbag.iter() {
             match attr {
-                Attribute::X => self.x.set(parse("x", value, LengthDir::Horizontal)?),
-                Attribute::Y => self.y.set(parse("y", value, LengthDir::Vertical)?),
+                Attribute::X => self.x.set(attr.parse(value, LengthDir::Horizontal)?),
+                Attribute::Y => self.y.set(attr.parse(value, LengthDir::Vertical)?),
                 Attribute::Dx => self
                     .dx
-                    .set(parse("dx", value, LengthDir::Horizontal).map(Some)?),
+                    .set(attr.parse(value, LengthDir::Horizontal).map(Some)?),
                 Attribute::Dy => self
                     .dy
-                    .set(parse("dy", value, LengthDir::Vertical).map(Some)?),
+                    .set(attr.parse(value, LengthDir::Vertical).map(Some)?),
                 _ => (),
             }
         }
@@ -573,7 +709,7 @@ fn extract_chars_children_to_chunks_recursively(
 }
 
 impl NodeTrait for NodeTRef {
-    fn set_atts(&self, _: &RsvgNode, _: *const RsvgHandle, pbag: &PropertyBag<'_>) -> NodeResult {
+    fn set_atts(&self, _: &RsvgNode, pbag: &PropertyBag<'_>) -> NodeResult {
         for (attr, value) in pbag.iter() {
             match attr {
                 Attribute::XlinkHref => {
@@ -629,21 +765,21 @@ impl NodeTSpan {
 }
 
 impl NodeTrait for NodeTSpan {
-    fn set_atts(&self, _: &RsvgNode, _: *const RsvgHandle, pbag: &PropertyBag<'_>) -> NodeResult {
+    fn set_atts(&self, _: &RsvgNode, pbag: &PropertyBag<'_>) -> NodeResult {
         for (attr, value) in pbag.iter() {
             match attr {
                 Attribute::X => self
                     .x
-                    .set(parse("x", value, LengthDir::Horizontal).map(Some)?),
+                    .set(attr.parse(value, LengthDir::Horizontal).map(Some)?),
                 Attribute::Y => self
                     .y
-                    .set(parse("y", value, LengthDir::Vertical).map(Some)?),
+                    .set(attr.parse(value, LengthDir::Vertical).map(Some)?),
                 Attribute::Dx => self
                     .dx
-                    .set(parse("dx", value, LengthDir::Horizontal).map(Some)?),
+                    .set(attr.parse(value, LengthDir::Horizontal).map(Some)?),
                 Attribute::Dy => self
                     .dy
-                    .set(parse("dy", value, LengthDir::Vertical).map(Some)?),
+                    .set(attr.parse(value, LengthDir::Vertical).map(Some)?),
                 _ => (),
             }
         }
@@ -656,6 +792,23 @@ fn to_pango_units(v: f64) -> i32 {
     (v * f64::from(pango::SCALE) + 0.5) as i32
 }
 
+impl<'a> From<&'a XmlLang> for pango::Language {
+    fn from(l: &'a XmlLang) -> pango::Language {
+        pango::Language::from_string(&l.0)
+    }
+}
+
+impl From<TextRendering> for cairo::Antialias {
+    fn from(tr: TextRendering) -> cairo::Antialias {
+        match tr {
+            TextRendering::Auto
+            | TextRendering::OptimizeLegibility
+            | TextRendering::GeometricPrecision => cairo::Antialias::Default,
+            TextRendering::OptimizeSpeed => cairo::Antialias::None,
+        }
+    }
+}
+
 impl From<FontStyle> for pango::Style {
     fn from(s: FontStyle) -> pango::Style {
         match s {
@@ -753,19 +906,55 @@ impl From<WritingMode> for pango::Gravity {
     }
 }
 
+fn get_pango_context(cr: &cairo::Context, is_testing: bool) -> pango::Context {
+    let font_map = pangocairo::FontMap::get_default().unwrap();
+    let context = font_map.create_context().unwrap();
+    pangocairo::functions::update_context(&cr, &context);
+
+    // Pango says this about pango_cairo_context_set_resolution():
+    //
+    //     Sets the resolution for the context. This is a scale factor between
+    //     points specified in a #PangoFontDescription and Cairo units. The
+    //     default value is 96, meaning that a 10 point font will be 13
+    //     units high. (10 * 96. / 72. = 13.3).
+    //
+    // I.e. Pango font sizes in a PangoFontDescription are in *points*, not pixels.
+    // However, we are normalizing everything to userspace units, which amount to
+    // pixels.  So, we will use 72.0 here to make Pango not apply any further scaling
+    // to the size values we give it.
+    //
+    // An alternative would be to divide our font sizes by (dpi_y / 72) to effectively
+    // cancel out Pango's scaling, but it's probably better to deal with Pango-isms
+    // right here, instead of spreading them out through our Length normalization
+    // code.
+    pangocairo::functions::context_set_resolution(&context, 72.0);
+
+    if is_testing {
+        let mut options = cairo::FontOptions::new();
+
+        options.set_antialias(cairo::Antialias::Gray);
+        options.set_hint_style(cairo::enums::HintStyle::Full);
+        options.set_hint_metrics(cairo::enums::HintMetrics::On);
+
+        pangocairo::functions::context_set_font_options(&context, &options);
+    }
+
+    context
+}
+
 fn create_pango_layout(
     draw_ctx: &DrawingCtx,
     values: &ComputedValues,
     text: &str,
 ) -> pango::Layout {
-    let pango_context = draw_ctx.get_pango_context();
+    let cr = draw_ctx.get_cairo_context();
+    let pango_context = get_pango_context(&cr, draw_ctx.is_testing());
 
     // See the construction of the XmlLang property
     // We use "" there as the default value; this means that the language is not set.
     // If the language *is* set, we can use it here.
     if !values.xml_lang.0.is_empty() {
-        let pango_lang = pango::Language::from_string(&values.xml_lang.0);
-        pango_context.set_language(&pango_lang);
+        pango_context.set_language(&pango::Language::from(&values.xml_lang));
     }
 
     pango_context.set_base_gravity(pango::Gravity::from(values.writing_mode));
@@ -785,15 +974,10 @@ fn create_pango_layout(
     }
 
     let mut font_desc = pango_context.get_font_description().unwrap();
-
     font_desc.set_family(&(values.font_family.0).0);
-
     font_desc.set_style(pango::Style::from(values.font_style));
-
     font_desc.set_variant(pango::Variant::from(values.font_variant));
-
     font_desc.set_weight(pango::Weight::from(values.font_weight.0));
-
     font_desc.set_stretch(pango::Stretch::from(values.font_stretch));
 
     let params = draw_ctx.get_view_params();
@@ -823,9 +1007,7 @@ fn create_pango_layout(
     }
 
     layout.set_attributes(&attr_list);
-
     layout.set_alignment(pango::Alignment::from(values.direction));
-
     layout.set_text(text);
 
     layout
